package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.Assignment;
import org.eclipse.jdt.internal.compiler.ast.CastExpression;
import org.eclipse.jdt.internal.compiler.ast.CompoundAssignment;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.IntLiteral;
import org.eclipse.jdt.internal.compiler.ast.NameReference;
import org.eclipse.jdt.internal.compiler.ast.QualifiedNameReference;
import org.eclipse.jdt.internal.compiler.ast.QualifiedSuperReference;
import org.eclipse.jdt.internal.compiler.ast.Reference;
import org.eclipse.jdt.internal.compiler.ast.SingleNameReference;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.flow.UnconditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.BaseTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.InferenceContext18;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
import org.eclipse.jdt.internal.compiler.lookup.ProblemFieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.VariableBinding;

public class FieldReference extends Reference implements InvocationSite {
  public static final int READ = 0;
  
  public static final int WRITE = 1;
  
  public Expression receiver;
  
  public char[] token;
  
  public FieldBinding binding;
  
  public MethodBinding[] syntheticAccessors;
  
  public long nameSourcePosition;
  
  public TypeBinding actualReceiverType;
  
  public TypeBinding genericCast;
  
  public FieldReference(char[] source, long pos) {
    this.token = source;
    this.nameSourcePosition = pos;
    this.sourceStart = (int)(pos >>> 32L);
    this.sourceEnd = (int)(pos & 0xFFFFFFFFL);
    this.bits |= 0x1;
  }
  
  public FlowInfo analyseAssignment(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, Assignment assignment, boolean isCompound) {
    if (isCompound) {
      if (this.binding.isBlankFinal() && 
        this.receiver.isThis() && 
        currentScope.needBlankFinalFieldInitializationCheck(this.binding)) {
        FlowInfo fieldInits = flowContext.getInitsForFinalBlankInitializationCheck(this.binding.declaringClass.original(), flowInfo);
        if (!fieldInits.isDefinitelyAssigned(this.binding))
          currentScope.problemReporter().uninitializedBlankFinalField(this.binding, (ASTNode)this); 
      } 
      manageSyntheticAccessIfNecessary(currentScope, flowInfo, true);
    } 
    UnconditionalFlowInfo unconditionalFlowInfo = 
      this.receiver
      .analyseCode(currentScope, flowContext, flowInfo, !this.binding.isStatic())
      .unconditionalInits();
    if (assignment.expression != null)
      unconditionalFlowInfo = 
        assignment
        .expression
        .analyseCode(currentScope, flowContext, (FlowInfo)unconditionalFlowInfo)
        .unconditionalInits(); 
    manageSyntheticAccessIfNecessary(currentScope, (FlowInfo)unconditionalFlowInfo, false);
    if (this.binding.isFinal()) {
      if (this.binding.isBlankFinal() && 
        !isCompound && 
        this.receiver.isThis() && 
        !(this.receiver instanceof org.eclipse.jdt.internal.compiler.ast.QualifiedThisReference) && (
        this.receiver.bits & 0x1FE00000) == 0 && 
        currentScope.allowBlankFinalFieldAssignment(this.binding)) {
        if (unconditionalFlowInfo.isPotentiallyAssigned(this.binding)) {
          currentScope.problemReporter().duplicateInitializationOfBlankFinalField(
              this.binding, 
              this);
        } else {
          flowContext.recordSettingFinal((VariableBinding)this.binding, this, (FlowInfo)unconditionalFlowInfo);
        } 
        unconditionalFlowInfo.markAsDefinitelyAssigned(this.binding);
      } else {
        currentScope.problemReporter().cannotAssignToFinalField(this.binding, (ASTNode)this);
      } 
    } else if (this.binding.isNonNull() || this.binding.type.isTypeVariable()) {
      if (!isCompound && 
        this.receiver.isThis() && 
        !(this.receiver instanceof org.eclipse.jdt.internal.compiler.ast.QualifiedThisReference) && 
        TypeBinding.equalsEquals(this.receiver.resolvedType, (TypeBinding)this.binding.declaringClass) && (
        this.receiver.bits & 0x1FE00000) == 0)
        unconditionalFlowInfo.markAsDefinitelyAssigned(this.binding); 
    } 
    return (FlowInfo)unconditionalFlowInfo;
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    return analyseCode(currentScope, flowContext, flowInfo, true);
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, boolean valueRequired) {
    boolean nonStatic = !this.binding.isStatic();
    this.receiver.analyseCode(currentScope, flowContext, flowInfo, nonStatic);
    if (nonStatic)
      this.receiver.checkNPE(currentScope, flowContext, flowInfo, 1); 
    if (valueRequired || (currentScope.compilerOptions()).complianceLevel >= 3145728L)
      manageSyntheticAccessIfNecessary(currentScope, flowInfo, true); 
    if ((currentScope.compilerOptions()).complianceLevel >= 3342336L) {
      FieldBinding fieldBinding = this.binding;
      if (this.receiver.isThis() && fieldBinding.isBlankFinal() && currentScope.needBlankFinalFieldInitializationCheck(fieldBinding)) {
        FlowInfo fieldInits = flowContext.getInitsForFinalBlankInitializationCheck(fieldBinding.declaringClass.original(), flowInfo);
        if (!fieldInits.isDefinitelyAssigned(fieldBinding))
          currentScope.problemReporter().uninitializedBlankFinalField(fieldBinding, (ASTNode)this); 
      } 
    } 
    return flowInfo;
  }
  
  public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, int ttlForFieldCheck) {
    if (flowContext.isNullcheckedFieldAccess(this))
      return true; 
    return checkNullableFieldDereference((Scope)scope, this.binding, this.nameSourcePosition, flowContext, ttlForFieldCheck);
  }
  
  public void computeConversion(Scope scope, TypeBinding runtimeTimeType, TypeBinding compileTimeType) {
    if (runtimeTimeType == null || compileTimeType == null)
      return; 
    if (this.binding != null && this.binding.isValidBinding()) {
      FieldBinding originalBinding = this.binding.original();
      TypeBinding originalType = originalBinding.type;
      if (originalType.leafComponentType().isTypeVariable()) {
        TypeBinding targetType = (!compileTimeType.isBaseType() && runtimeTimeType.isBaseType()) ? 
          compileTimeType : 
          runtimeTimeType;
        this.genericCast = originalBinding.type.genericCast(targetType);
        if (this.genericCast instanceof ReferenceBinding) {
          ReferenceBinding referenceCast = (ReferenceBinding)this.genericCast;
          if (!referenceCast.canBeSeenBy(scope))
            scope.problemReporter().invalidType((ASTNode)this, 
                (TypeBinding)new ProblemReferenceBinding(
                  CharOperation.splitOn('.', referenceCast.shortReadableName()), 
                  referenceCast, 
                  2)); 
        } 
      } 
    } 
    super.computeConversion(scope, runtimeTimeType, compileTimeType);
  }
  
  public FieldBinding fieldBinding() {
    return this.binding;
  }
  
  public void generateAssignment(BlockScope currentScope, CodeStream codeStream, Assignment assignment, boolean valueRequired) {
    int pc = codeStream.position;
    FieldBinding codegenBinding = this.binding.original();
    this.receiver.generateCode(currentScope, codeStream, !codegenBinding.isStatic());
    codeStream.recordPositionsFrom(pc, this.sourceStart);
    assignment.expression.generateCode(currentScope, codeStream, true);
    fieldStore((Scope)currentScope, codeStream, codegenBinding, (this.syntheticAccessors == null) ? null : this.syntheticAccessors[1], this.actualReceiverType, this.receiver.isImplicitThis(), valueRequired);
    if (valueRequired)
      codeStream.generateImplicitConversion(assignment.implicitConversion); 
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
    int pc = codeStream.position;
    if (this.constant != Constant.NotAConstant) {
      if (valueRequired)
        codeStream.generateConstant(this.constant, this.implicitConversion); 
      codeStream.recordPositionsFrom(pc, this.sourceStart);
      return;
    } 
    FieldBinding codegenBinding = this.binding.original();
    boolean isStatic = codegenBinding.isStatic();
    boolean isThisReceiver = this.receiver instanceof org.eclipse.jdt.internal.compiler.ast.ThisReference;
    Constant fieldConstant = codegenBinding.constant();
    if (fieldConstant != Constant.NotAConstant) {
      if (!isThisReceiver) {
        this.receiver.generateCode(currentScope, codeStream, !isStatic);
        if (!isStatic) {
          codeStream.invokeObjectGetClass();
          codeStream.pop();
        } 
      } 
      if (valueRequired)
        codeStream.generateConstant(fieldConstant, this.implicitConversion); 
      codeStream.recordPositionsFrom(pc, this.sourceStart);
      return;
    } 
    if (valueRequired || (
      !isThisReceiver && (currentScope.compilerOptions()).complianceLevel >= 3145728L) || (
      this.implicitConversion & 0x400) != 0 || 
      this.genericCast != null) {
      this.receiver.generateCode(currentScope, codeStream, !isStatic);
      if ((this.bits & 0x40000) != 0)
        codeStream.checkcast(this.actualReceiverType); 
      pc = codeStream.position;
      if (codegenBinding.declaringClass == null) {
        codeStream.arraylength();
        if (valueRequired) {
          codeStream.generateImplicitConversion(this.implicitConversion);
        } else {
          codeStream.pop();
        } 
      } else {
        if (this.syntheticAccessors == null || this.syntheticAccessors[0] == null) {
          TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
          if (isStatic) {
            codeStream.fieldAccess((byte)-78, codegenBinding, constantPoolDeclaringClass);
          } else {
            codeStream.fieldAccess((byte)-76, codegenBinding, constantPoolDeclaringClass);
          } 
        } else {
          codeStream.invoke((byte)-72, this.syntheticAccessors[0], null);
        } 
        if (this.genericCast != null)
          codeStream.checkcast(this.genericCast); 
        if (valueRequired) {
          codeStream.generateImplicitConversion(this.implicitConversion);
        } else {
          boolean isUnboxing = ((this.implicitConversion & 0x400) != 0);
          if (isUnboxing)
            codeStream.generateImplicitConversion(this.implicitConversion); 
          switch (isUnboxing ? (postConversionType((Scope)currentScope)).id : codegenBinding.type.id) {
            case 7:
            case 8:
              codeStream.pop2();
              break;
            default:
              codeStream.pop();
              break;
          } 
        } 
      } 
    } else if (isThisReceiver) {
      if (isStatic)
        if (TypeBinding.notEquals((TypeBinding)(this.binding.original()).declaringClass, this.actualReceiverType.erasure())) {
          MethodBinding accessor = (this.syntheticAccessors == null) ? null : this.syntheticAccessors[0];
          if (accessor == null) {
            TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
            codeStream.fieldAccess((byte)-78, codegenBinding, constantPoolDeclaringClass);
          } else {
            codeStream.invoke((byte)-72, accessor, null);
          } 
          switch (codegenBinding.type.id) {
            case 7:
            case 8:
              codeStream.pop2();
              break;
            default:
              codeStream.pop();
              break;
          } 
        }  
    } else {
      this.receiver.generateCode(currentScope, codeStream, !isStatic);
      if (!isStatic) {
        codeStream.invokeObjectGetClass();
        codeStream.pop();
      } 
    } 
    codeStream.recordPositionsFrom(pc, this.sourceEnd);
  }
  
  public void generateCompoundAssignment(BlockScope currentScope, CodeStream codeStream, Expression expression, int operator, int assignmentImplicitConversion, boolean valueRequired) {
    reportOnlyUselesslyReadPrivateField(currentScope, this.binding, valueRequired);
    FieldBinding codegenBinding = this.binding.original();
    boolean isStatic;
    this.receiver.generateCode(currentScope, codeStream, !(isStatic = codegenBinding.isStatic()));
    if (isStatic) {
      if (this.syntheticAccessors == null || this.syntheticAccessors[0] == null) {
        TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
        codeStream.fieldAccess((byte)-78, codegenBinding, constantPoolDeclaringClass);
      } else {
        codeStream.invoke((byte)-72, this.syntheticAccessors[0], null);
      } 
    } else {
      codeStream.dup();
      if (this.syntheticAccessors == null || this.syntheticAccessors[0] == null) {
        TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
        codeStream.fieldAccess((byte)-76, codegenBinding, constantPoolDeclaringClass);
      } else {
        codeStream.invoke((byte)-72, this.syntheticAccessors[0], null);
      } 
    } 
    int operationTypeID;
    switch (operationTypeID = (this.implicitConversion & 0xFF) >> 4) {
      case 0:
      case 1:
      case 11:
        codeStream.generateStringConcatenationAppend(currentScope, null, expression);
        break;
      default:
        if (this.genericCast != null)
          codeStream.checkcast(this.genericCast); 
        codeStream.generateImplicitConversion(this.implicitConversion);
        if (expression == IntLiteral.One) {
          codeStream.generateConstant(expression.constant, this.implicitConversion);
        } else {
          expression.generateCode(currentScope, codeStream, true);
        } 
        codeStream.sendOperator(operator, operationTypeID);
        codeStream.generateImplicitConversion(assignmentImplicitConversion);
        break;
    } 
    fieldStore((Scope)currentScope, codeStream, codegenBinding, (this.syntheticAccessors == null) ? null : this.syntheticAccessors[1], this.actualReceiverType, this.receiver.isImplicitThis(), valueRequired);
  }
  
  public void generatePostIncrement(BlockScope currentScope, CodeStream codeStream, CompoundAssignment postIncrement, boolean valueRequired) {
    TypeBinding operandType;
    reportOnlyUselesslyReadPrivateField(currentScope, this.binding, valueRequired);
    FieldBinding codegenBinding = this.binding.original();
    boolean isStatic;
    this.receiver.generateCode(currentScope, codeStream, !(isStatic = codegenBinding.isStatic()));
    if (isStatic) {
      if (this.syntheticAccessors == null || this.syntheticAccessors[0] == null) {
        TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
        codeStream.fieldAccess((byte)-78, codegenBinding, constantPoolDeclaringClass);
      } else {
        codeStream.invoke((byte)-72, this.syntheticAccessors[0], null);
      } 
    } else {
      codeStream.dup();
      if (this.syntheticAccessors == null || this.syntheticAccessors[0] == null) {
        TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
        codeStream.fieldAccess((byte)-76, codegenBinding, constantPoolDeclaringClass);
      } else {
        codeStream.invoke((byte)-72, this.syntheticAccessors[0], null);
      } 
    } 
    if (this.genericCast != null) {
      codeStream.checkcast(this.genericCast);
      operandType = this.genericCast;
    } else {
      operandType = codegenBinding.type;
    } 
    if (valueRequired)
      if (isStatic) {
        switch (operandType.id) {
          case 7:
          case 8:
            codeStream.dup2();
            break;
          default:
            codeStream.dup();
            break;
        } 
      } else {
        switch (operandType.id) {
          case 7:
          case 8:
            codeStream.dup2_x1();
            break;
          default:
            codeStream.dup_x1();
            break;
        } 
      }  
    codeStream.generateImplicitConversion(this.implicitConversion);
    codeStream.generateConstant(
        postIncrement.expression.constant, 
        this.implicitConversion);
    codeStream.sendOperator(postIncrement.operator, this.implicitConversion & 0xF);
    codeStream.generateImplicitConversion(
        postIncrement.preAssignImplicitConversion);
    fieldStore((Scope)currentScope, codeStream, codegenBinding, (this.syntheticAccessors == null) ? null : this.syntheticAccessors[1], this.actualReceiverType, this.receiver.isImplicitThis(), false);
  }
  
  public TypeBinding[] genericTypeArguments() {
    return null;
  }
  
  public InferenceContext18 freshInferenceContext(Scope scope) {
    return null;
  }
  
  public boolean isEquivalent(Reference reference) {
    if (this.receiver.isThis() && !(this.receiver instanceof org.eclipse.jdt.internal.compiler.ast.QualifiedThisReference)) {
      char[] otherToken = null;
      if (reference instanceof SingleNameReference) {
        otherToken = ((SingleNameReference)reference).token;
      } else if (reference instanceof org.eclipse.jdt.internal.compiler.ast.FieldReference) {
        org.eclipse.jdt.internal.compiler.ast.FieldReference fr = (org.eclipse.jdt.internal.compiler.ast.FieldReference)reference;
        if (fr.receiver.isThis() && !(fr.receiver instanceof org.eclipse.jdt.internal.compiler.ast.QualifiedThisReference))
          otherToken = fr.token; 
      } 
      return (otherToken != null && CharOperation.equals(this.token, otherToken));
    } 
    char[][] thisTokens = getThisFieldTokens(1);
    if (thisTokens == null)
      return false; 
    char[][] otherTokens = null;
    if (reference instanceof org.eclipse.jdt.internal.compiler.ast.FieldReference) {
      otherTokens = ((org.eclipse.jdt.internal.compiler.ast.FieldReference)reference).getThisFieldTokens(1);
    } else if (reference instanceof QualifiedNameReference) {
      if (((QualifiedNameReference)reference).binding instanceof org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding)
        return false; 
      otherTokens = ((QualifiedNameReference)reference).tokens;
    } 
    return CharOperation.equals(thisTokens, otherTokens);
  }
  
  private char[][] getThisFieldTokens(int nestingCount) {
    char[][] result = null;
    if (this.receiver.isThis() && !(this.receiver instanceof org.eclipse.jdt.internal.compiler.ast.QualifiedThisReference)) {
      result = new char[nestingCount][];
      result[0] = this.token;
    } else if (this.receiver instanceof org.eclipse.jdt.internal.compiler.ast.FieldReference) {
      result = ((org.eclipse.jdt.internal.compiler.ast.FieldReference)this.receiver).getThisFieldTokens(nestingCount + 1);
      if (result != null)
        result[result.length - nestingCount] = this.token; 
    } 
    return result;
  }
  
  public boolean isSuperAccess() {
    return this.receiver.isSuper();
  }
  
  public boolean isQualifiedSuper() {
    return this.receiver.isQualifiedSuper();
  }
  
  public boolean isTypeAccess() {
    return (this.receiver != null && this.receiver.isTypeReference());
  }
  
  public FieldBinding lastFieldBinding() {
    return this.binding;
  }
  
  public void manageSyntheticAccessIfNecessary(BlockScope currentScope, FlowInfo flowInfo, boolean isReadAccess) {
    if ((flowInfo.tagBits & 0x1) != 0)
      return; 
    FieldBinding codegenBinding = this.binding.original();
    if (this.binding.isPrivate()) {
      if (TypeBinding.notEquals((TypeBinding)currentScope.enclosingSourceType(), (TypeBinding)codegenBinding.declaringClass) && 
        this.binding.constant((Scope)currentScope) == Constant.NotAConstant) {
        if (this.syntheticAccessors == null)
          this.syntheticAccessors = new MethodBinding[2]; 
        this.syntheticAccessors[isReadAccess ? 0 : 1] = 
          (MethodBinding)((SourceTypeBinding)codegenBinding.declaringClass).addSyntheticMethod(codegenBinding, isReadAccess, false);
        currentScope.problemReporter().needToEmulateFieldAccess(codegenBinding, (ASTNode)this, isReadAccess);
        return;
      } 
    } else {
      if (this.receiver instanceof QualifiedSuperReference) {
        SourceTypeBinding destinationType = (SourceTypeBinding)((QualifiedSuperReference)this.receiver).currentCompatibleType;
        if (this.syntheticAccessors == null)
          this.syntheticAccessors = new MethodBinding[2]; 
        this.syntheticAccessors[isReadAccess ? 0 : 1] = (MethodBinding)destinationType.addSyntheticMethod(codegenBinding, isReadAccess, isSuperAccess());
        currentScope.problemReporter().needToEmulateFieldAccess(codegenBinding, (ASTNode)this, isReadAccess);
        return;
      } 
      if (this.binding.isProtected())
        if ((this.bits & 0x1FE0) != 0) {
          SourceTypeBinding enclosingSourceType;
          if (this.binding.declaringClass.getPackage() != (
            enclosingSourceType = currentScope.enclosingSourceType()).getPackage()) {
            SourceTypeBinding currentCompatibleType = 
              (SourceTypeBinding)enclosingSourceType.enclosingTypeAt((
                this.bits & 0x1FE0) >> 5);
            if (this.syntheticAccessors == null)
              this.syntheticAccessors = new MethodBinding[2]; 
            this.syntheticAccessors[isReadAccess ? 0 : 1] = (MethodBinding)currentCompatibleType.addSyntheticMethod(codegenBinding, isReadAccess, isSuperAccess());
            currentScope.problemReporter().needToEmulateFieldAccess(codegenBinding, (ASTNode)this, isReadAccess);
            return;
          } 
        }  
    } 
  }
  
  public Constant optimizedBooleanConstant() {
    if (this.resolvedType == null)
      return Constant.NotAConstant; 
    switch (this.resolvedType.id) {
      case 5:
      case 33:
        return (this.constant != Constant.NotAConstant) ? this.constant : this.binding.constant();
    } 
    return Constant.NotAConstant;
  }
  
  public TypeBinding postConversionType(Scope scope) {
    BaseTypeBinding baseTypeBinding;
    TypeBinding typeBinding1, convertedType = this.resolvedType;
    if (this.genericCast != null)
      convertedType = this.genericCast; 
    int runtimeType = (this.implicitConversion & 0xFF) >> 4;
    switch (runtimeType) {
      case 5:
        baseTypeBinding = TypeBinding.BOOLEAN;
        break;
      case 3:
        baseTypeBinding = TypeBinding.BYTE;
        break;
      case 4:
        baseTypeBinding = TypeBinding.SHORT;
        break;
      case 2:
        baseTypeBinding = TypeBinding.CHAR;
        break;
      case 10:
        baseTypeBinding = TypeBinding.INT;
        break;
      case 9:
        baseTypeBinding = TypeBinding.FLOAT;
        break;
      case 7:
        baseTypeBinding = TypeBinding.LONG;
        break;
      case 8:
        baseTypeBinding = TypeBinding.DOUBLE;
        break;
    } 
    if ((this.implicitConversion & 0x200) != 0)
      typeBinding1 = scope.environment().computeBoxingType((TypeBinding)baseTypeBinding); 
    return typeBinding1;
  }
  
  public StringBuffer printExpression(int indent, StringBuffer output) {
    return this.receiver.printExpression(0, output).append('.').append(this.token);
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    boolean receiverCast = false;
    if (this.receiver instanceof CastExpression) {
      this.receiver.bits |= 0x20;
      receiverCast = true;
    } 
    this.actualReceiverType = this.receiver.resolveType(scope);
    if (this.actualReceiverType == null) {
      this.constant = Constant.NotAConstant;
      return null;
    } 
    if (receiverCast)
      if (TypeBinding.equalsEquals(((CastExpression)this.receiver).expression.resolvedType, this.actualReceiverType))
        scope.problemReporter().unnecessaryCast((CastExpression)this.receiver);  
    FieldBinding fieldBinding = this.binding = scope.getField(this.actualReceiverType, this.token, this);
    if (!fieldBinding.isValidBinding()) {
      this.constant = Constant.NotAConstant;
      if (this.receiver.resolvedType instanceof ProblemReferenceBinding)
        return null; 
      ReferenceBinding declaringClass = fieldBinding.declaringClass;
      boolean avoidSecondary = (declaringClass != null && 
        declaringClass.isAnonymousType() && 
        declaringClass.superclass() instanceof org.eclipse.jdt.internal.compiler.lookup.MissingTypeBinding);
      if (!avoidSecondary)
        scope.problemReporter().invalidField(this, this.actualReceiverType); 
      if (fieldBinding instanceof ProblemFieldBinding) {
        ProblemFieldBinding problemFieldBinding = (ProblemFieldBinding)fieldBinding;
        FieldBinding closestMatch = problemFieldBinding.closestMatch;
        switch (problemFieldBinding.problemId()) {
          case 2:
          case 5:
          case 6:
          case 7:
            if (closestMatch != null)
              fieldBinding = closestMatch; 
            break;
        } 
      } 
      if (!fieldBinding.isValidBinding())
        return null; 
    } 
    TypeBinding oldReceiverType = this.actualReceiverType;
    this.actualReceiverType = this.actualReceiverType.getErasureCompatibleType((TypeBinding)fieldBinding.declaringClass);
    this.receiver.computeConversion((Scope)scope, this.actualReceiverType, this.actualReceiverType);
    if (TypeBinding.notEquals(this.actualReceiverType, oldReceiverType) && TypeBinding.notEquals(this.receiver.postConversionType((Scope)scope), this.actualReceiverType))
      this.bits |= 0x40000; 
    if (isFieldUseDeprecated(fieldBinding, (Scope)scope, this.bits))
      scope.problemReporter().deprecatedField(fieldBinding, (ASTNode)this); 
    boolean isImplicitThisRcv = this.receiver.isImplicitThis();
    this.constant = isImplicitThisRcv ? fieldBinding.constant((Scope)scope) : Constant.NotAConstant;
    if (fieldBinding.isStatic()) {
      if (!isImplicitThisRcv && (
        !(this.receiver instanceof NameReference) || (
        ((NameReference)this.receiver).bits & 0x4) == 0))
        scope.problemReporter().nonStaticAccessToStaticField((ASTNode)this, fieldBinding); 
      ReferenceBinding declaringClass = this.binding.declaringClass;
      if (!isImplicitThisRcv && 
        TypeBinding.notEquals((TypeBinding)declaringClass, this.actualReceiverType) && 
        declaringClass.canBeSeenBy((Scope)scope))
        scope.problemReporter().indirectAccessToStaticField((ASTNode)this, fieldBinding); 
      if (declaringClass.isEnum()) {
        MethodScope methodScope = scope.methodScope();
        SourceTypeBinding sourceType = scope.enclosingSourceType();
        if (this.constant == Constant.NotAConstant && 
          !methodScope.isStatic && (
          TypeBinding.equalsEquals((TypeBinding)sourceType, (TypeBinding)declaringClass) || TypeBinding.equalsEquals((TypeBinding)sourceType.superclass, (TypeBinding)declaringClass)) && 
          methodScope.isInsideInitializerOrConstructor())
          scope.problemReporter().enumStaticFieldUsedDuringInitialization(this.binding, (ASTNode)this); 
      } 
    } 
    TypeBinding fieldType = fieldBinding.type;
    if (fieldType != null) {
      if ((this.bits & 0x2000) == 0)
        fieldType = fieldType.capture((Scope)scope, this.sourceStart, this.sourceEnd); 
      this.resolvedType = fieldType;
      if ((fieldType.tagBits & 0x80L) != 0L) {
        scope.problemReporter().invalidType((ASTNode)this, fieldType);
        return null;
      } 
    } 
    return fieldType;
  }
  
  public void setActualReceiverType(ReferenceBinding receiverType) {
    this.actualReceiverType = (TypeBinding)receiverType;
  }
  
  public void setDepth(int depth) {
    this.bits &= 0xFFFFE01F;
    if (depth > 0)
      this.bits |= (depth & 0xFF) << 5; 
  }
  
  public void setFieldIndex(int index) {}
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope))
      this.receiver.traverse(visitor, scope); 
    visitor.endVisit(this, scope);
  }
  
  public VariableBinding nullAnnotatedVariableBinding(boolean supportTypeAnnotations) {
    if (this.binding != null && (
      supportTypeAnnotations || (
      this.binding.tagBits & 0x180000000000000L) != 0L))
      return (VariableBinding)this.binding; 
    return null;
  }
}
